import numpy as np

# X = (heures de révision, heures de sommeil, petit dejeuner), y = note à l'examen
data = np.array(([3, 8], [1, 5], [4, 6], [2, 9], [1, 7],[3, 5],[2, 6]), dtype=float) # données d'entrées
y = np.array(([15], [6], [12], [9]), dtype=float) # donnée de sortie: La note à l'examen

# Normalisation des données avec la données la plus grande
data = data/np.amax(data, axis=0) # Normalisation des données par rapport à leur max
y = y/20 # Normalisation de la note par rapport à 20 (la note maximale est 20/20)

# Séparation des données en données d'entrainement et données à prédire
X = np.split(data, [4])[0] # training data
x_predicted = np.split(data, [4])[1] # data to predict output after training step



class neural_network(object):
# Initiation de la classe
  def __init__(self):
  # Parametres de taille des 3 couche du réseau
    self.inputSize = 2
    self.outputSize = 1
    self.hiddenSize = 3

  # Poids
    # (3x2) Matrice des poids de la couche d'entrée à la couche abstraite
    self.W1 = np.random.randn(self.inputSize, self.hiddenSize) 
    # (3x1) Matrice des poids de la couche abstraite à la couche de sortie
    self.W2 = np.random.randn(self.hiddenSize, self.outputSize)

# Propagation à travers le réseau de neurones
  def forward(self, X):
    # Produit matriciel en X (données d'entrées) et le premier jeu de poids (matrice W1 3x3)
    self.z = np.dot(X, self.W1) 
    # Application de la fonction d'activation
    self.z2 = self.sigmoid(self.z) 
    # Produit matriciel la couche abstraite et le deuxième jeu de poids (matrice W2 3x1)
    self.z3 = np.dot(self.z2, self.W2)
    # Application de la fonction d'activiation et obtention de la note    
    o = self.sigmoid(self.z3)
    return o

# Fonction d'activation
  def sigmoid(self, s):
    return 1/(1+np.exp(-s))

# Dérivée de la fonction d'activation (Utile pour la retro propagation)
  def sigmoidPrime(self, s):
    return s * (1 - s)

# Retro propagation
  def backward(self, X, y, o):
    # Erreur de sortie (écart entre la note prédite et la note réelle)
    self.o_error = y - o 
    # Application de la dérivée de la sigmoid sur l'erreur pour commencer la retro propagation
    self.o_delta = self.o_error*self.sigmoidPrime(o) 

    # Evalutation de la contribution des poids sur l'erreur (pour tenter de corriger les poids)
    self.z2_error = self.o_delta.dot(self.W2.T) 
    # Application de la dérivée de la sigmoid sur la couche abstraite
    # La retro propagation continue
    self.z2_delta = self.z2_error*self.sigmoidPrime(self.z2)

    # Ajustement des poids W1 (Couche d'entrée --> couche abstraite)
    self.W1 += X.T.dot(self.z2_delta) 
    # Ajustement des poids W2 (couche abstraite --> couche de sortie)
    self.W2 += self.z2.T.dot(self.o_delta)

  def train(self, X, y):
    o = self.forward(X)
    self.backward(X, y, o)

  def predict(self):
    print("Note prédite: ")
    print("Entrée (Normalisée):")
    print(str(x_predicted))
    print("Note:")
    print(str(self.forward(x_predicted)))

nn = neural_network()
# ENTRAINEMENT DU RESEAU DE NEURONES 1000 fois pour affiner les poids
for i in range(1000): 
  print("# " + str(i) + "\n")
  print("Entrée (Normalisée): \n" + str(X))
  print("Note réelle: \n" + str(y))
  print("Note prédite: \n" + str(nn.forward(X)))
  # Erreur (somme des carrés des ecarts)mean squared error
  print("Erreur: \n" + str(np.mean(np.square(y - nn.forward(X)))))
  print("\n")
  nn.train(X, y)

nn.predict()